package org.apidesign.apifest08.currency;

import java.util.Currency;

/**
 * This is the skeleton class for your API. You need to make it public, so it is accessible to your client code
 * (currently in Task1Test.java) file.
 * <p>
 * Feel free to create additional classes or rename this one, just keep all the API and its implementation in this
 * package. Do not spread it outside to other packages.
 */
public class Convertor {

  private Currency currency1;
  private Currency currency2;

  private Convertor(Currency currency1, Currency currency2) {
    this.currency1 = currency1;
    this.currency2 = currency2;
  }

  /**
   * Creates new instance of convertor.
   * @param currency1
   *          one of the currencies we want to convert to/from
   * @param currency2
   *          the other currency
   * @return new instance of convertor
   */
  public static Convertor getConvertorInstance(Currency currency1, Currency currency2) {
    if (currency1 == null || currency2 == null)
      throw new ConvertorException("None of the currencies should be null!!!");
    return new Convertor(currency1, currency2);
  }

  /**
   * Converts selected amout of selected currency to other currency of this convertor instance.
   * @param amount
   *          amount to convert
   * @param actualCurrency
   *          currency of this amount
   * @return converted amount
   */
  public double convert(double amount, Currency actualCurrency) {
    ExchangeRate exchangeRate = null;

    if (actualCurrency == null) {
      throw new ConvertorException("Selected currency is null!!!");
    }

    if (currency1.getSymbol().equals(actualCurrency.getSymbol())
        || currency2.getSymbol().equals(actualCurrency.getSymbol())) {
      exchangeRate = getExchangeRate(actualCurrency);
    } else {
      throw new ConvertorException("This convertor could not be used for selected currency("
          + actualCurrency.getCurrencyCode() + ") " + this + "!!!");
    }

    return countResult(exchangeRate, amount);
  }

  private double countResult(ExchangeRate exchangeRate, double amount) {
    return amount * exchangeRate.getRate() / exchangeRate.getUnit();
  }

  /**
   * Decides the direction of conversion and returns instance of actual exchange rate.
   * @param actualCurrency
   *          actual currency we want to convert
   * @return actual exchange rate of this convertor for selected currency
   */
  private ExchangeRate getExchangeRate(Currency actualCurrency) {
    ExchangeRate exchangeRate = null;

    if (actualCurrency.getCurrencyCode().equals(currency1.getCurrencyCode())) {
      exchangeRate = getExchangeRateInstance(actualCurrency, currency2);
    } else if (actualCurrency.getCurrencyCode().equals(currency2.getCurrencyCode())) {
      exchangeRate = getExchangeRateInstance(actualCurrency, currency1);
    }

    return exchangeRate;
  }

  /**
   * Returns actual exchange rate between original and destination currency.
   * @param originalCurrency
   *          original currency, from which we want to convert
   * @param newCurrency
   *          destination currency, to which we want to convert
   * @return exchange rate between this two currencies
   */
  private ExchangeRate getExchangeRateInstance(Currency originalCurrency, Currency newCurrency) {
    if ("USD".equals(originalCurrency.getCurrencyCode()) && "CZK".equals(newCurrency.getCurrencyCode())) {
      return new ExchangeRate(originalCurrency, newCurrency, 17d, 1d);
    } else if ("CZK".equals(originalCurrency.getCurrencyCode()) && "USD".equals(newCurrency.getCurrencyCode())) {
      return new ExchangeRate(originalCurrency, newCurrency, 1d / 17d, 1d);
    } else if ("SKK".equals(originalCurrency.getCurrencyCode()) && "CZK".equals(newCurrency.getCurrencyCode())) {
      return new ExchangeRate(originalCurrency, newCurrency, 80d, 100d);
    } else if ("CZK".equals(originalCurrency.getCurrencyCode()) && "SKK".equals(newCurrency.getCurrencyCode())) {
      return new ExchangeRate(originalCurrency, newCurrency, 100d / (80d / 100d), 100d);
    } else {
      throw new ConvertorException("Defined currencies don't have secified rates ["
          + originalCurrency.getCurrencyCode() + ", " + newCurrency.getCurrencyCode() + "]!!!");
    }
  }

  public String toString() {
    return "Converter [" + currency1.getCurrencyCode() + ", " + currency2.getCurrencyCode() + "]";
  }
}
